%--------------------------------------------------------------------------------------------------%
% Copyright (C) <2011> by <Ralph Smith (rsmith@eos.ncsu.edu), Zhengzheng Hu (zhu4@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%
%
% This function uses the strain data to find the initial estimates for rate dependent parameters:
% gamma and tau
% User needs to supply two input field values E1 and E2 that are constant during
% which creep behavior is observed. 
%           
%--------------------------------------------------------------------------------------------------%
function [gamma,tau_90]=solve_taugamma(data1,data2,params,scales,non_opt_params)

% Get the two constant field values
E1=sum(data1.E)/length(data1.E);
E2=sum(data2.E)/length(data2.E);


% Select three sample points to calculate d2 and d3
step1 = data1.step;

N11 = step1+1; 
dt1 = data1.T(N11+step1)-data1.T(N11);
d12=data1.S(N11+step1)-data1.S(N11);
d13=data1.S(N11+2*step1)-data1.S(N11+step1);


step2 = data2.step;

N21 = floor(length(data2.S)/3);
dt2 = data2.T(N21+step2)-data2.T(N21);
d22=data2.S(N21+step2)-data2.S(N21);
d23=data2.S(N21+2*step2)-data2.S(N21+step2);

if(abs(d13)>abs(d12))
    disp('Error, d13>d12, reselect points');
elseif(abs(d23)>abs(d22))
    disp('Error, d23>d22, reselect points');
end

%--------------------------------------------------------------------------------------------------%
% 
% figure
% subplot(1,2,1)
% hold on
% plot(data1.T,data1.S,'r--','LineWidth',2)
% plot(data1.T(N11-step1),data1.S(N11-step1),'k.','MarkerSize',20)
% plot(data1.T(N11),data1.S(N11),'k.','MarkerSize',20)
% plot(data1.T(N11+step1),data1.S(N11+step1),'k.','MarkerSize',20)
% plot(data1.T(N11+2*step1),data1.S(N11+2*step1),'k.','MarkerSize',20)
% set(gca,'FontSize',16)
% xlabel('Time (s)')
% ylabel('Strain (%)')
% subplot(1,2,2)
% hold on
% plot(data2.T,data2.S,'r--','LineWidth',2)
% plot(data2.T(N21-step2),data2.S(N21-step2),'k.','MarkerSize',20)
% plot(data2.T(N21),data2.S(N21),'k.','MarkerSize',20)
% plot(data2.T(N21+step2),data2.S(N21+step2),'k.','MarkerSize',20)
% plot(data2.T(N21+2*step2),data2.S(N21+2*step2),'k.','MarkerSize',20)
% set(gca,'FontSize',16)
% xlabel('Time (s)')
% ylabel('Strain (%)')
%
%--------------------------------------------------------------------------------------------------%

fc_bar = non_opt_params.fc_bar;
dg0 = 0.25*fc_bar;

P_R_p = params.P_R_p*scales.P_R_p;
P_R_m = -P_R_p;
P_R_90 = params.P_R_90*scales.P_R_90;

EPS_R_p = params.EPS_R_p*scales.EPS_R_p;
EPS_R_m = EPS_R_p;
EPS_R_90 = params.EPS_R_90*scales.EPS_R_90;

chi_p = params.chi_p*scales.chi_p;
d_p = params.d_p*scales.d_p;
s_p = params.s_p*scales.s_p;

chi_m = chi_p;
d_m = -d_p;
s_m = s_p;

chi_90 = chi_p;
s_90 = s_p;

d_90 = params.d_90*scales.d_90;


%--------------------------------------------------------------------------------------------------%
E= E1;
SIGMA = data1.Sigma;

P_m_p = P_R_p+chi_p*E+d_p.*SIGMA;
EPS_m_p = EPS_R_p+d_p*E+s_p.*SIGMA;

P_m_m = P_R_m+chi_m*E+d_m.*SIGMA;
EPS_m_m = EPS_R_m+d_m*E+s_m.*SIGMA;

P_m_90 = P_R_90+chi_90*E+d_90.*SIGMA;
EPS_m_90 = EPS_R_90+d_90*E+s_90.*SIGMA;

G_m_p = -0.5*E.*(P_m_p+P_R_p)-0.5*SIGMA.*(EPS_m_p+EPS_R_p);
G_m_m = -0.5*E.*(P_m_m+P_R_m)-0.5*SIGMA.*(EPS_m_m+EPS_R_m);
G_m_90 = -0.5*E.*(P_m_90+P_R_90)-0.5*SIGMA.*(EPS_m_90+EPS_R_90);

f_p_90 = G_m_p-G_m_90;
f_90_m = G_m_90-G_m_m;
f_m_90 = -f_90_m;
f_90_p = -f_p_90;

dg_p_90 = dg0*(1-f_p_90./fc_bar).^2.*(f_p_90<fc_bar)+0*(f_p_90>=fc_bar);
dg_90_p = dg0*(1-f_90_p./fc_bar).^2.*(f_90_p<fc_bar)+0*(f_90_p>=fc_bar);
dg_90_m = dg0*(1-f_90_m./fc_bar).^2.*(f_90_m<fc_bar)+0*(f_90_m>=fc_bar);
dg_m_90 = dg0*(1-f_m_90./fc_bar).^2.*(f_m_90<fc_bar)+0*(f_m_90>=fc_bar);


if (myfun(5e-3,E, dg_p_90,dg_90_p,dg_90_m,dg_m_90)-1>1e-8)
    disp('S1 is not in the correct region, relect an input field!')
    return
end
tau_90 = dt1/log(d12/d13);

%--------------------------------------------------------------------------------------------------%

E= E2;
SIGMA = data2.Sigma;

P_m_p = P_R_p+chi_p*E+d_p.*SIGMA;
EPS_m_p = EPS_R_p+d_p*E+s_p.*SIGMA;

P_m_m = P_R_m+chi_m*E+d_m.*SIGMA;
EPS_m_m = EPS_R_m+d_m*E+s_m.*SIGMA;

P_m_90 = P_R_90+chi_90*E+d_90.*SIGMA;
EPS_m_90 = EPS_R_90+d_90*E+s_90.*SIGMA;

G_m_p = -0.5*E.*(P_m_p+P_R_p)-0.5*SIGMA.*(EPS_m_p+EPS_R_p);
G_m_m = -0.5*E.*(P_m_m+P_R_m)-0.5*SIGMA.*(EPS_m_m+EPS_R_m);
G_m_90 = -0.5*E.*(P_m_90+P_R_90)-0.5*SIGMA.*(EPS_m_90+EPS_R_90);

f_p_90 = G_m_p-G_m_90;
f_90_m = G_m_90-G_m_m;
f_m_90 = -f_90_m;
f_90_p = -f_p_90;

dg_p_90 = dg0*(1-f_p_90./fc_bar).^2.*(f_p_90<fc_bar)+0*(f_p_90>=fc_bar);
dg_90_p = dg0*(1-f_90_p./fc_bar).^2.*(f_90_p<fc_bar)+0*(f_90_p>=fc_bar);
dg_90_m = dg0*(1-f_90_m./fc_bar).^2.*(f_90_m<fc_bar)+0*(f_90_m>=fc_bar);
dg_m_90 = dg0*(1-f_m_90./fc_bar).^2.*(f_m_90<fc_bar)+0*(f_m_90>=fc_bar);

gamma = fzero(@(gamma)( myfun(gamma,E,dg_p_90,dg_90_p,dg_90_m,dg_m_90)-tau_90/dt2*log(d22/d23)),5e-3);


function f=myfun(x,E, dg_p_90,dg_90_p,dg_90_m,dg_m_90)
f=likelihood(x,dg_90_m).*(E<0)+likelihood(x,dg_90_p).*(E>0);

function f=likelihood(x,dg)
f = exp(-dg*x);




